from Products.validation.interfaces import ivalidator

class FrecklesValidator:

    __implements__ = (ivalidator,)

    def __init__(self, name):
        self.name = name

    def __call__(self, value, *args, **kwargs):
        self.lines = {'x'      : [],
                      'y'      : [],
                      'radius' : [],
                      'red'    : [],
                      'green'  : [],
                      'blue'   : []}
                  
        # setup and check for item count errors:

        line_errors = []
        line_count = 1
        for freckle_string in value:
            x = ""
            y = ""
            radius = ""
            red = ""
            green = ""
            blue = ""
            try:
                items = eval('[%s]' % freckle_string)
            except:
                return "Mixed numeric and text input somewhere on line %d" % line_count
            if len(items) != 6:
                line_errors.append("%d" % line_count)
            else:
                x,y,radius,red,green,blue = items
                self.lines['x'].append(x)
                self.lines['y'].append(y)
                self.lines['radius'].append(radius)
                self.lines['red'].append(red)
                self.lines['green'].append(green)
                self.lines['blue'].append(blue)
            line_count += 1
        if len(line_errors) > 0:
            return "Wrong number of items on lines: %s" % line_errors

        # check for basic conversion errors:

        error_message = ""
        errors = None

        errors = self.checkForXErrors()
        if errors:
            error_message = "X (field 1) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForYErrors()
        if errors:
            error_message = "Y (field 2) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForRadiusErrors()
        if errors:
            error_message = "Radius (field 3) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForRedErrors()
        if errors:
            error_message = "Red (field 4) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForGreenErrors()
        if errors:
            error_message = "Green (field 5) error on lines: %s" % str(errors)
            return error_message

        errors = self.checkForBlueErrors()
        if errors:
            error_message = "Blue (field 6) error on lines: %s" % str(errors)
            return error_message

        # check for start and end time range errors:
        error_message = ""

        # check for color range errors:
        errors = self.checkForColorRangeErrors()
        if errors:
            error_message = "One or more color values out of range (0-255) on lines: %s" % str(errors)
            return error_message

        return 1

    def checkForXErrors(self):
        line_errors = []
        count = 1
        for x in self.lines['x']:
            try:
                int(x)
            except:
                line_errors.append(count)
            if int(x) < 0:
                line_errors.append(count)
            count += 1
        if len(line_errors) > 0:
            return line_errors
        return None

    def checkForYErrors(self):
        line_errors = []
        count = 1
        for y in self.lines['y']:
            try:
                int(y)
            except:
                line_errors.append(count)
            if int(y) < 0:
                line_errors.append(count)
            count += 1
        if len(line_errors) > 0:
            return line_errors
        return None

    def checkForRadiusErrors(self):
        line_errors = []
        count = 1
        for radius in self.lines['radius']:
            try:
                int(radius)
            except:
                line_errors.append(count)
            count += 1
        if len(line_errors) > 0:
            return line_errors
        return None

    def checkForColorErrors(self, color_name):
        line_errors = []
        count = 1
        for color in self.lines[color_name]:
            try:
                int(color)
            except:
                line_errors.append(count)
        if len(line_errors) > 0:
            return line_errors
        return None

    def checkForRedErrors(self):
        return self.checkForColorErrors('red')

    def checkForGreenErrors(self):
        return self.checkForColorErrors('green')

    def checkForBlueErrors(self):
        return self.checkForColorErrors('blue')

    def checkForColorRangeErrors(self):
        line_errors = []
        for i in range(len(self.lines['red'])):
            red = int(self.lines['red'][i])
            green = int(self.lines['green'][i])
            blue = int(self.lines['blue'][i])

            # must be between 0 and 255
            if red < 0 or red > 255 or green < 0 or green > 255 or blue < 0 or blue > 255:
                line_errors.append(i+1)
        if len(line_errors) > 0:
            return line_errors
        return None
