# Version 0.1
#
#  SmileyFace is copyright 2005 Jeffrey G. Pittman.
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
#  MA 02111-1307 USA

import sys, csv, string, cStringIO

from Products.SmileyFace.utils import getrlcolor
from Products.SmileyFace.utils import colorstr
from Products.SmileyFace.utils import wordWrap

try :
    from reportlab.graphics.shapes import *
    from reportlab.graphics.shapes import Circle as RLCircle
    from reportlab.graphics.shapes import Ellipse as RLEllipse
    from reportlab.graphics.shapes import PolyLine as RLPolyLine
    from reportlab.graphics.charts.textlabels import Label
except ImportError :
    from reportlab.graphics.shapes import *
    from reportlab.graphics.shapes import Circle as RLCircle
    from reportlab.graphics.shapes import Ellipse as RLEllipse
    from reportlab.graphics.shapes import PolyLine as RLPolyLine
    from reportlab.graphics.charts.textlabels import Label

def yUp(y):
    return 400.0 - float(y)

# The following are wrappers around the ReportLab shapes, 
# for the rl_draw methods, and also provide strarray methods
# for directly emitting SVG.
#
class Line:
    def __init__(self,start,end,color,width):
        self.start = start #xy tuple
        self.end = end     #xy tuple
        self.color = color
        self.width = width
        return

    def rl_draw(self, drawing):
        drawing.add(Line(self.start[0],self.start[1],self.end[0],self.end[1],getrlcolor(self.color[0],self.color[1],self.color[2]),strokeLineCap=0))

    def strarray(self):
        return ["    <line x1=\"%d\" y1=\"%d\" x2=\"%d\" y2=\"%d\" style=\"stroke:%s;stroke-width:%d\" />\n" %\
                (self.start[0],self.start[1],self.end[0],self.end[1],colorstr(self.color),self.width)]

class Circle:
    def __init__(self,center,radius,color):
        self.center = center #xy tuple
        self.radius = radius 
        self.color = color   #rgb tuple in range(0,256)
        return

    def rl_draw(self, drawing):
        drawing.add(Circle(self.center[0],self.center[1],self.radius,fillColor=getrlcolor(self.color[0],self.color[1],self.color[2])))

    def strarray(self):
        return ["    <circle cx=\"%d\" cy=\"%d\" r=\"%d\" " %\
                (self.center[0],self.center[1],self.radius),
                "style=\"fill:%s;\" />\n" % colorstr(self.color)]

class Rectangle:
    def __init__(self,origin,height,width,color):
        self.origin = origin
        self.height = height
        self.width = width
        self.color = color
        return

    def rl_draw(self, drawing):
        drawing.add(Rect(self.origin[0],yUp(self.origin[1]+self.height),self.width,self.height,fillColor=getrlcolor(self.color[0],self.color[1],self.color[2])))

    def strarray(self):
        return ["    <rect x=\"%d\" y=\"%d\" height=\"%d\" " %\
                (self.origin[0],self.origin[1],self.height),
                "width=\"%d\" style=\"fill:%s;\" />\n" %\
                (self.width,colorstr(self.color))]

class IDRectangle:
    def __init__(self,indentation,origin,id,height,width,color,begin,end,desc):
        self.indentation = indentation
        self.origin = origin
        self.id = id #id.replace(' ','_')
        self.height = height
        self.width = width
        self.color = color
        self.begin = begin
        self.end = end
        self.desc1 = " "
        self.desc2 = " "
        self.desc3 = " "
        desc_lines = wordWrap(desc,80).split('\n')
        if len(desc_lines) == 1:
            self.desc1 = desc_lines[0]
        elif len(desc_lines) == 2:
            self.desc1 = desc_lines[0]
            self.desc2 = desc_lines[1]
        elif len(desc_lines) == 3:
            self.desc1 = desc_lines[0]
            self.desc2 = desc_lines[1]
            self.desc3 = desc_lines[2]

    def rl_draw(self, drawing):
        drawing.add(Rect(self.origin[0],
                         yUp(self.origin[1]+self.height),
                         self.width,
                         self.height,
                         fillColor=getrlcolor(self.color[0],self.color[1],self.color[2])))
        #label = Label()
        #label.setOrigin(self.origin[0]+self.width+5, yUp(self.origin[1]+(self.height/2.0)))
        #label.fillColor = getrlcolor(self.color[0],self.color[1],self.color[2])
        #label.fontSize = 12
        #label.setText("%s (%2.1f to %2.1f Ma)" % (self.id,self.begin,self.end))
        #drawing.add(label)

    def strarray(self):
        var = ["%s<rect id=\"%s\" x=\"%d\" y=\"%d\" height=\"%d\" " %\
               (self.indentation,self.id.replace(' ','_'),self.origin[0],self.origin[1],self.height),
               "width=\"%d\" style=\"stroke-width:0.2;fill:%s;\"><title>%s (%2.1f to %2.1f Ma)</title></rect>\n" %\
               (self.width,colorstr(self.color),self.id,self.begin,self.end)]
#        if len(self.desc1) > 0:
#            var += "    <desc1>%s</desc1>"  % self.desc1
#        if len(self.desc2) > 0:
#            var += "    <desc2>%s</desc2>"  % self.desc2
#        if len(self.desc3) > 0:
#            var += "    <desc3>%s</desc3>"  % self.desc3
#        var += "</rect>\n"
        return var

class Text:
    def __init__(self,origin,text,color,size=24):
        self.origin = origin
        self.text = text
        self.size = size
        self.color = color
        return

    def rl_draw(self, drawing):
        label = Label()
        label.setOrigin(self.origin[0], yUp(self.origin[1]))
        label.fillColor = getrlcolor(self.color[0],self.color[1],self.color[2])
        label.fontSize = self.size
        label.setText(self.text)
        drawing.add(label)

    def strarray(self):
        return ["    <text x=\"%d\" y=\"%d\" font-size=\"%d\" fill=\"%s\">" %\
                (self.origin[0],self.origin[1],self.size,colorstr(self.color)),
                "%s" % self.text,
                "</text>\n"]
        
class TransformedText:
    def __init__(self,origin,text,color,size=24,translation_x=0,translation_y=0,rotation=0):
        self.origin = origin
        self.text = text
        self.color = color
        self.size = size
        self.translation_x = translation_x
        self.translation_y = translation_y
        self.rotation = rotation
        return

    def strarray(self):
        return ["    <g transform=\"translate(%d,%d)\">\n" %\
                (self.translation_x,self.translation_y),
                "        <g transform=\"rotate(%d)\">\n" %\
                (self.rotation),
                "            <text x=\"%d\" y=\"%d\" font-size=\"%d\" fill=\"%s\">" %\
                (self.origin[0],self.origin[1],self.size,colorstr(self.color)),
                "%s" % self.text,
                "</text>\n",
                "        </g>\n",
                "    </g>\n"]
        
class SmileyFaceDrawingEngine:
    """
    The basic structure of SVG emitting methods is a modification of:
    http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/325823,
    whereby the strarray() method emits a list of lines that are
    added to a growing SVG file.

    SVG help at www.carto.net was also very useful.    
    """
    def __init__(self,
                 freckles,
                 name="smileyface.svg",
                 height=400,
                 width=400,
                 x_offset = 10,
                 y_offset = 10):
        self.freckles = freckles
        self.width = width
        self.height = height
        self.x_offset = x_offset
        self.y_offset = y_offset
        self.name = name

    def rl_draw(self, drawing):
        self.fillColor = colors.yellow
        self.strokeColor = colors.black

        # general widget bits
        s = float(self.width - (self.x_offset * 2))  # abbreviate as we will use this a lot

        # SmileyFace specific bits
        drawing.add(RLCircle(self.x_offset+(s/2), self.y_offset+(s/2), (s/2) - self.x_offset,
                             fillColor=self.fillColor, strokeColor=self.strokeColor,
                             strokeWidth=max(s/38.,2.)))

        for i in (1,2):
            drawing.add(RLEllipse(self.x_offset+(s/3)*i,self.y_offset+(s/3)*2, s/30, s/10,
                                  fillColor=self.fillColor, strokeColor = self.strokeColor,
                                  strokeWidth=max(s/38.,2.)))

        # A message from reportlab kit (from signsandsymbols.py):
        #   calculate a pointslist for the mouth
        #   THIS IS A HACK! - don't use if there is a 'shapes.Arc'
        centerx=self.x_offset+(s/2)
        centery=self.y_offset+(s/2)
        radius=s/3
        yradius = radius
        xradius = radius
        startangledegrees=200
        endangledegrees=340
        degreedelta = 1
        pointslist = []
        a = pointslist.append
        from math import sin, cos, pi
        degreestoradians = pi/180.0
        radiansdelta = degreedelta*degreestoradians
        startangle = startangledegrees*degreestoradians
        endangle = endangledegrees*degreestoradians
        while endangle<startangle:
              endangle = endangle+2*pi
        angle = startangle
        while angle<endangle:
            x = centerx + cos(angle)*radius
            y = centery + sin(angle)*yradius
            a(x); a(y)
            angle = angle+radiansdelta

        # make the mouth
        smile = RLPolyLine(pointslist,
                           fillColor = self.strokeColor,
                           strokeColor = self.strokeColor,
                           strokeWidth = max(s/38.,2.))
        drawing.add(smile)

    def getOpenerSection(self):
        opener_section = \
          ["<?xml version=\"1.0\"?>\n",
           "<svg  id=\"smileyface\" xmlns=\"http://www.w3.org/2000/svg\" xmlns:xlink=\"http://www.w3.org/1999/xlink\" onload=\"Init(evt)\" onmousemove=\"GetTrueCoords(evt); ShowTooltip(evt, true)\" onmouseout=\"ShowTooltip(evt, false)\" height=\"%d\" width=\"%d\">\n" % (self.height,self.width)]
        return opener_section
                      
    def getDefsSection(self):
        defs_section = ["    <defs>\n"]
        defs_section += self.getCSS()
        defs_section += self.getEcmascript()
        defs_section += ["    </defs>\n"]
        return defs_section
                    
    def write_ecmascript(self,filename=None):
        if filename:
            self.ecmascriptname = filename
        else:
            self.ecmascriptname = self.name + ".js"
        file = open(self.ecmascriptname,'w')
        file.writelines(self.getEcmascript())
        file.close()

    def getCSS(self):
        css = \
          ["\n",
           "        <style type=\"text/css\">\n",
           "            <![CDATA[\n",
           "                body {font-family:Arial,Helvetica,Geneva,sans-serif;}\n",
           "                .str0 {\n",
           "                    stroke:#000000;\n",
           "                    stroke-width:0.1\n",
           "                }\n",
           "            ]]>\n",
           "        </style>\n",
           "\n"]
        return css

    def getEcmascript(self):
        """
        Note: at one time there were three description lines in the mouseover.
        These are commented out. This could be useful to have more info
            in the tooltip.
        """
        ecmascript = \
            ["        <script language=\"Javascript\"><![CDATA[\n",
             "        // This script from http://svg-whiz.com/svg/Tooltip.svg\n",
             "        var SVGDocument = null;\n",
             "        var SVGRoot = null;\n",
             "        var SVGViewBox = null;\n",
             "        var svgns = 'http://www.w3.org/2000/svg';\n",
             "        var xlinkns = 'http://www.w3.org/1999/xlink';\n",
             "        var toolTip = null;\n",
             "        var TrueCoords = null;\n",
             "        var tipBox = null;\n",
             "        var tipText = null;\n",
             "        var tipTitle = null;\n",
             #"        var tipDesc1 = null;\n",
             #"        var tipDesc2 = null;\n",
             #"        var tipDesc3 = null;\n",
             "\n",
             "        var lastElement = null;\n",
             "        var titleText = '';\n",
             #"        var titleDesc1 = '';\n",
             #"        var titleDesc2 = '';\n",
             #"        var titleDesc3 = '';\n",
             "        function Init(evt)\n",
             "        {\n",
             "        SVGDocument = evt.target.ownerDocument;\n",
             "        SVGRoot = document.documentElement;\n",
             "        TrueCoords = SVGRoot.createSVGPoint();\n",
             "        \n",
             "        toolTip = SVGDocument.getElementById('ToolTip');\n",
             "        tipBox = SVGDocument.getElementById('TipBox');\n",
             "        tipText = SVGDocument.getElementById('TipText');\n",
             "        tipTitle = SVGDocument.getElementById('TipTitle');\n",
             #"        tipDesc1 = SVGDocument.getElementById('tipDesc1');\n",
             #"        tipDesc2 = SVGDocument.getElementById('tipDesc2');\n",
             #"        tipDesc3 = SVGDocument.getElementById('tipDesc3');\n",
             "        };\n",
             "\n",
             "        function GetTrueCoords(evt)\n",
             "        {\n",
             "           // find the current zoom level and pan setting, and adjust the reported\n",
             "           //    mouse position accordingly\n",
             "           var newScale = SVGRoot.currentScale;\n",
             "           var translation = SVGRoot.currentTranslate;\n",
             "           TrueCoords.x = (evt.clientX - translation.x)/newScale;\n",
             "           TrueCoords.y = (evt.clientY - translation.y)/newScale;\n",
             "        };\n",
             "\n",
             "        function ShowTooltip(evt, turnOn)\n",
             "        {\n",
             "           try\n",
             "           {\n",
             "              if (!evt || !turnOn)\n",
             "              {\n",
             "                 toolTip.setAttributeNS(null, 'display', 'none');\n",
             "              }\n",
             "              else\n",
             "              {\n",
             "                 var tipScale = 1/SVGRoot.currentScale;\n",
             "                 var textWidth = 0;\n",
             "                 var tspanWidth = 0;\n",
             "                 var boxHeight = 20;\n",
             "\n",
             "                 tipBox.setAttributeNS(null, 'transform', 'scale(' + tipScale + ',' + tipScale + ')' );\n",
             "                 tipText.setAttributeNS(null, 'transform', 'scale(' + tipScale + ',' + tipScale + ')' );\n",
             "\n",
             "                 var targetElement = evt.target;\n",
             "\n",
             "                 if ( lastElement != targetElement )\n",
             "                 {\n",
             "                    var targetTitle = targetElement.getElementsByTagName('title').item(0);\n",
             "                    if ( targetTitle )\n",
             "                    {\n",
             "                       titleText = targetTitle.firstChild.nodeValue;\n",
             "                       tipTitle.firstChild.nodeValue = titleText;\n",
             "                    }\n",
             "\n",
             #"                    var targetDesc1 = targetElement.getElementsByTagName('desc1').item(0);\n",
             #"                    if ( targetDesc1 )\n",
             #"                    {\n",
             #"                       titleDesc1 = targetDesc1.firstChild.nodeValue;\n",
             #"                       tipDesc1.firstChild.nodeValue = titleDesc1;\n",
             #"                    }\n",
             #"\n",
             #"                    var targetDesc2 = targetElement.getElementsByTagName('desc2').item(0);\n",
             #"                    if ( targetDesc2 )\n",
             #"                    {\n",
             #"                       titleDesc2 = targetDesc2.firstChild.nodeValue;\n",
             #"                       tipDesc2.firstChild.nodeValue = titleDesc2;\n",
             #"                    }\n",
             #"\n",
             #"                    var targetDesc3 = targetElement.getElementsByTagName('desc3').item(0);\n",
             #"                    if ( targetDesc3 )\n",
             #"                    {\n",
             #"                       titleDesc3 = targetDesc3.firstChild.nodeValue;\n",
             #"                       tipDesc3.firstChild.nodeValue = titleDesc3;\n",
             #"                    }\n",
             "                 }\n",
             "\n",
             "                 var xPos = TrueCoords.x + (10 * tipScale);\n",
             "                 var yPos = TrueCoords.y + (10 * tipScale);\n",
             "\n",
             "                 //return rectangle around object as SVGRect object\n",
             "                 var outline = tipText.getBBox();\n",
             "                 tipBox.setAttributeNS(null, 'width', Number(outline.width) + 10);\n",
             "                 tipBox.setAttributeNS(null, 'height', Number(outline.height) + 10);\n",
             "\n",
             "                 toolTip.setAttributeNS(null, 'transform', 'translate(' + xPos + ',' + yPos + ')');\n",
             "                 toolTip.setAttributeNS(null, 'display', 'inline');\n",
             "              }\n",
             "           }\n",
             "           catch(er){}\n",
             "         };\n",
             "        ]]></script>\n"]
        return ecmascript

    def getSmileyFaceSection(self):
        self.fillColor = (255,255,0)
        self.strokeColor = (0,0,0)
        self.strokeWidth = 20 

        # general widget bits
        s = float(self.width - (self.x_offset * 2))  # abbreviate as we will use this a lot

        smiley_svg = \
            ["    <circle cx=\"%d\" cy=\"%d\" r=\"%d\" " %\
                  (self.x_offset+(s/2),yUp(self.y_offset+(s/2)), s/2),
             "style=\"fill:%s;stroke:%s;stroke-width:%d\" />\n" % (colorstr(self.fillColor),colorstr(self.strokeColor),self.strokeWidth)]


        for i in (1,2):
            smiley_svg += ["    <ellipse cx=\"%d\" cy=\"%d\" rx=\"%d\" ry=\"%d\" " %\
                                (self.x_offset+(s/3)*i,yUp(self.y_offset+(s/3)*2), s/30, s/10),
                           "style=\"stroke:%s; stroke-width:20\" />\n" % colorstr(self.strokeColor)]

        # A message from reportlab kit (from signsandsymbols.py):
        #     calculate a pointslist for the mouth
        #     THIS IS A HACK! - don't use if there is a 'shapes.Arc'
        # 
        # With SVG, the origin is at the upper left corner, and y values
        # decrease as you move upward. Normally we use uUp in rl_draw
        # methods, but as this smiley face routine was written for
        # reportlab, we need to reverse y.
        centerx=self.x_offset+(s/2)
        centery=self.y_offset+(s/2)
        radius=s/3
        yradius = radius
        xradius = radius
        startangledegrees=200
        endangledegrees=340
        degreedelta = 1
        pointslist = []
        a = pointslist.append
        from math import sin, cos, pi
        degreestoradians = pi/180.0
        radiansdelta = degreedelta*degreestoradians
        startangle = startangledegrees*degreestoradians
        endangle = endangledegrees*degreestoradians
        while endangle<startangle:
              endangle = endangle+2*pi
        angle = startangle
        while angle<endangle:
            x = centerx + cos(angle)*radius
            y = centery + sin(angle)*yradius
            a(x); a(yUp(y))
            angle = angle+radiansdelta

        # make the mouth
        smiley_svg += ["<polyline points=\"%s\" fill=\"none\" style=\"stroke:%s;stroke-width:%d\"/>\n" %\
                           (self.getPointsListString(pointslist),colorstr(self.strokeColor),max(s/38.,self.strokeWidth))]

        return smiley_svg

    def getPointsListString(self, pointslist):
        points_list_string = ""
        i = 0
        while i < len(pointslist)-1:
            points_list_string += "%d,%d " % (pointslist[i],pointslist[i+1])
            i += 2
        return points_list_string

    def getFrecklesSection(self):
        var = []
        var += ["    <g id=\"freckles\">\n"]
        for freckle in self.freckles:
            var += ["    <circle cx=\"%d\" cy=\"%d\" r=\"%d\" " %\
                        (freckle.x,yUp(freckle.y),freckle.radius),
                        "style=\"fill:%s;\" />\n" % colorstr(freckle.color)]
        var += ["    </g>\n"]
        return var

    def rl_drawLabel(self, drawing, text_x, text_y, angle, boxAnchor, fontSize, fontColor, text):
        label = Label()
        label.setOrigin(text_x, yUp(text_y))
        label.angle = angle
        label.boxAnchor = boxAnchor
        label.fontSize = fontSize
        label.fillColor = getrlcolor(fontColor[0],fontColor[1],fontColor[2])
        label.setText(text)
        drawing.add(label)

    def getBottomPopupSection(self):
        bottom_svg = \
          ["    <g id='ToolTip' opacity='0.8' display='none' pointer-events='none'>\n",
           "        <rect id='TipBox' x='0' y='5' width='88' height='20' rx='2' ry='2' fill='white' stroke='black'/>\n",
           "        <text id='TipText' x='5' y='20' font-family='Arial' font-size='12'>\n",
           "            <tspan id='TipTitle' x='5' font-weight='bold'> </tspan>\n",
           #"         <tspan id='tipDesc1' x='5' dy='1.2em' fill='blue'> </tspan>\n",
           #"         <tspan id='tipDesc2' x='5' dy='1.2em' fill='blue'> </tspan>\n",
           #"         <tspan id='tipDesc3' x='5' dy='1.2em' fill='blue'> </tspan>\n",
           "        </text>\n",
           "    </g>\n"]
        return bottom_svg
    
    def getCloserSection(self):
        closer_section = ["</svg>\n"]
        return closer_section
        
    def getSVG(self):
        svg_code = self.getOpenerSection()
        svg_code += self.getDefsSection()
        svg_code += self.getSmileyFaceSection()
        svg_code += self.getFrecklesSection()
        svg_code += self.getBottomPopupSection()
        svg_code += self.getCloserSection()
        return svg_code

    def write_html(self,filename=None):
        if filename:
            self.htmlname = filename
        else:
            self.htmlname = self.name + ".html"
        file = open(self.htmlname,'w')
        file.writelines(self.getHTML(self.name))
        file.close()

    def getHTML(self, name):        
        var = \
            ["  <script type=\"text/javascript\">\n",
            #["<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">\n",
             #"<html>\n",
             #"<head>\n",
             #"  <meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n",
             #"  <meta http-equiv=\"imagetoolbar\" content=\"no\">\n",
             #"  <meta name=\"keywords\" content=\"geologic time scale svg scalable vector graphics\">\n",
             #"  <meta name=\"description\" content=\"SVG SmileyFace\">\n",
             #"  <style type=\"text/css\">\n",
             #"      body {background-color:#ffef95;font-family:Arial,Helvetica,Geneva,sans-serif;color:#B42D25;}\n",
             #"      .bigger {font-weight:bold;}\n",
             #"      .smaller {font-size:small;}\n",
             #"  </style>\n",
             "      function hilite_elem(checkbox, element_name){\n",
             "        // For each element, get the element's style object, then set\n",
             "        // its visibility according to the state of the checkbox.\n",
             "        var svgobj = document.embeds['smileyface'].getSVGDocument().getElementById(element_name);\n",
             "        if (!checkbox.checked){\n",
             "            // Hide layer.\n",
             "            svgobj.setAttributeNS(null,'visibility','hidden');\n",
             "        } else {\n",
             "            // Show layer.\n",
             "            svgobj.setAttributeNS(null,'visibility','visible');\n",
             "        }\n",
             "      }\n",
             "  </script>\n",
             #"  <title>SVG SmileyFace</title>\n",
             #"</head>\n",
             #"<body><a name=\"top\"></a>\n",
             "<h2>SVG SmileyFace</h2>\n",
             "      <table border=\"0\" cellpadding=\"8\" cellspacing=\"0\">\n",
             "        <tr>\n",
             "          <td align=\"center\" valign=\"top\">\n",
             "          <object width=\"600\" height=\"660\" data=\"%s.svg\" name=\"smileyface\" type=\"image/svg+xml\"/>\n" % name,
             "            <br>\n",
             "            <form name=\"hilite_form\" action=\"\">\n",
             "              <table border=\"0\" cellpadding=\"0\" cellspacing=\"2\">\n",
             "                <tr>\n",
             "                  <td colspan=\"2\">Visible Layers\n",
             "                  </td>\n",
             "                </tr>\n",
             "                <tr>\n",
             "                  <td><input type=\"checkbox\" value=\"\" onclick=\"hilite_elem(this,'bar_scale')\">&nbsp;Bar Scale</TD>\n",
             "                  <td><input type=\"checkbox\" value=\"\" onclick=\"hilite_elem(this,'header_labels')\">&nbsp;Header Labels</TD>\n",
             "                </tr>\n",
             "               </table>\n",
             "               <script type=\"text/javascript\">\n",
             "                 // Make sure all checkboxes are checked whenever the page is reloaded in the browser.\n",
             "                 for (var i = 0;  i < document.hilite_form.elements.length;  i++)\n",
             "                   if (document.hilite_form.elements[i].type == 'checkbox'){\n",
             "                     document.hilite_form.elements[i].checked = true;\n",
             "                   }  \n",
             "               </script>\n",
             "            </form>\n",
             "          </td>\n",
             "          <td valign=\"top\">\n",
             "            <p>Use mouse functions to rollover items and to change display:</p>\n",
             "            <ul>\n",
             "              <li><span class=\"bigger\">Rollover</span>: As you move the mouse over time units, time boundaries are shown.\n",
             "              <li><span class=\"bigger\">Layers</span>: Use the checkboxes at bottom to toggle items on and off.</li>\n",
             "            </ul>\n",
             "            <p>Use keyboard functions to change display:</p>\n",
             "            <ul>\n",
             "              <li><span class=\"bigger\">Zooming</span>: use the CTRL-Key (or Apple-Key) to drag a rectangle and zoom in.\n",
             "                  <br>Use the right mouse button (CTRL-Key&nbsp;+ mouse click on the Mac) to get a context menu, where you can choose step-by-step zooming and can go back to the original extent.</li>\n",
             "              <li><span class=\"bigger\">Zoom Out</span>: like above, but with SHIFT-Key pressed.</li>\n",
             "              <li><span class=\"bigger\">Panning</span>: Press the ALT-Key and move the mouse cursor while a \"hand-icon\" appears.</li>\n",
             "              <li><span class=\"bigger\">Anti-Aliasing</span>: May be turned on and off with the help of the context menu. (&quot;Higher Quality&quot;).</li>\n",
             "            </ul>\n",
             "          </td>\n",
             "        </tr>\n",
             "      </table>\n",
             "<br><br><hr>\n"]
             #"</body></html>\n"]
        return var

    def rl_drawFreckles(self, drawing):
        for freckle in self.freckles:
            drawing.add(RLCircle(freckle.x,freckle.y,freckle.radius,fillColor=getrlcolor(freckle.color[0],freckle.color[1],freckle.color[2])))

class Freckle:
    def __init__(self,x,y,radius,color):
        self.x = x
        self.y = y
        self.radius = radius
        self.color = color

class SmileyFaceDrawing:
    def __init__(self, image_types, height, width, x_offset, y_offset, scale_to_fit, simple_version, freckles_input):
        self.image_types = image_types
        self.height = height
        self.width = width
        self.x_offset = x_offset
        self.y_offset = y_offset
        self.scale_to_fit = scale_to_fit
        self.freckles_input = freckles_input

        name = "smileyface"

        self.smileyface = SmileyFaceDrawingEngine(self.parseFreckles(),
                                                  name,
                                                  height,
                                                  width,
                                                  x_offset,
                                                  y_offset)

    def parseFreckles(self):
        freckles = []
        for freckle_string in self.freckles_input:
            x,y,radius,red,green,blue = eval('[%s]' % freckle_string)
            freckles.append(Freckle(int(x),int(y),int(radius),(int(red),int(green),int(blue))))
        return freckles

    def getDrawing(self):
        self.drawing = Drawing(self.width, self.height)

        self.smileyface.rl_draw(self.drawing)
        
        self.smileyface.rl_drawFreckles(self.drawing)

        if self.scale_to_fit:
            scale_factor = float(self.height) / 400.0
            self.drawing.scale(scale_factor,scale_factor)
        else:
            # move origin down, because image 0,0 is bottom left of images (PNG, etc.)
            self.drawing.translate(0.0,-((self.y_offset+400.0)-self.height))

        return self.drawing

    def getSVG(self):
        return self.smileyface.getSVG()

    def getHTML(self, name):
        return self.smileyface.getHTML(name)
